#!/bin/sh
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

# Usage: This script is used to migrate the Storage Control DB2
#  database on Linux or AIX.  
# 
# The file uses one parameter, a filename that is sourced to 
#  set the following parameters (specified as PARAMETER=VALUE in the file
#  we read in):
#  
# MIGRATION_OPERATION=backup
#   MIGRATION_OPERATION can be either backup or restore
# DB_NAME=TPCDB
#   The name of the database we will backup or restore
# BACKUP_DIR=/tmp/test_backup
#   The file system location where we will write the backup file to backup or 
#    read the backup file to restore
# MIGRATE_INFO_FILE=/tmp/test_backup/backup.info
#   The filename we tell DB2 to write backup information to during the backup or
#    restore information during the restore.
# DB2_BIN_PATH=/opt/example/db2/bin
#   The path to the DB2 install to use for calling the db2 command. 
# DB2_AUTH_FILE=/tmp/test_backup/dbauth.info
#   The path to a file that specifies the authentication information
#    for DB2.  We split this from the file with the other parameters so we can
#    restrict access to it since the authentication information is in clear text.
# TIMESTAMP=20110420151448
#   The DB2 commands for restoring will use the most recent file by default.
#   Part of the filename is a timestamp.  If we want to specify a specific backup
#   to restore, we specify its timestamp.

# DB2_AUTH_FILE has the following parameters:
# DB_USER=dbUser
#   The DB2 user to use for SQL commands.
# DB_PASSWORD=dbPassword
#   The DB2 password to use for SQL commands.

MIGRATION_PARAM_FILE=$1
DIRHOME=$2

#Source the Migration Param File.
# i.e. /tmp/test_backup/sc_migration_params
. $MIGRATION_PARAM_FILE

#Set the DB2HOME path.
DB2HOME=$(cat /etc/passwd | grep $DB_USER | awk -F':' '{print $6}')
#Source the db2profile.
. $DB2HOME/sqllib/db2profile

#Set paths.
PATH=$DB2HOME/sqllib/bin:$PATH
LIBPATH=$DB2HOME/sqllib/lib64:$LIBPATH
LD_LIBRARY_PATH=$DB2HOME/sqllib/lib64:$LD_LIBRARY_PATH
export PATH 
export LIBPATH 
export LD_LIBRARY_PATH

MIGLOG=$DIRHOME/log/SCMigrate.log

echo "PATH:" $PATH >> $MIGLOG
echo "DB2HOME:" $DB2HOME >> $MIGLOG
echo "LIBPATH:" $LIBPATH >> $MIGLOG
echo "LD_LIBRARY_PATH:" $LD_LIBRARY_PATH >> $MIGLOG
echo "DB2INSTANCE:" $DB2INSTANCE >> $MIGLOG
echo "INSTANCENAME:" $INSTANCENAME >> $MIGLOG

# Read authentication information from a restricted file.
#. $DB2_AUTH_FILE

# If we want to specify a specific backup, use the timestamp for it.
#TIMESTAMP=$2

DB2_CMD=$DB2_BIN_PATH/db2

DB2OPTIONS='-o -z "$MIGRATE_INFO_FILE"'
export DB2OPTIONS=$DB2OPTIONS

migrationReturnCode="0"

#Remove temporary files.
function CleanUpFiles
{
	rm $MIGRATE_INFO_FILE
}

#Connect to the DB2 database and lock it so we have complete control.
function PrepareForMigration
{
	echo "$DB2_CMD connect TO $DB_NAME" >>  $MIGLOG
	$DB2_CMD CONNECT TO $DB_NAME USER $DB_USER USING $DB_PASSWORD;
	if [ "$?" -ne "0" ] ; then
	    exit 1
	fi


    if [ ! -d $BACKUP_DIR ] ; then
		mkdir -p $BACKUP_DIR 
    fi
    chmod 777 $BACKUP_DIR


	echo "Getting tablespace ids..." >> $MIGLOG
	
	touch $BACKUP_DIR/Tablespaces.txt  >> $MIGLOG 2>&1
	$DB2_CMD "select tbspace, tbspaceid from syscat.tablespaces" >> $MIGLOG
    $DB2_CMD "select tbspace, tbspaceid from syscat.tablespaces" >> $BACKUP_DIR/Tablespaces.txt 2>&1
	


	echo "Quiesce database ..." >> $MIGLOG
	$DB2_CMD QUIESCE DATABASE IMMEDIATE FORCE CONNECTIONS
	$DB2_CMD CONNECT RESET
}

#Run the DB2 database backup.
function BackupDatabase
{
	echo "Database backup directory: $BACKUP_DIR" >> $MIGLOG
	
	echo "Backing up database $DB_NAME ..." >> $MIGLOG
    
    BACKUPFILE=$($DB2_CMD BACKUP DATABASE $DB_NAME USER $DB_USER USING $DB_PASSWORD TO $BACKUP_DIR WITH 2 BUFFERS BUFFER 1024 PARALLELISM 1 COMPRESS WITHOUT PROMPTING) 
	migrationReturnCode=$?
	echo "Database backup finished with exit code $migrationReturnCode, check $MIGRATE_INFO_FILE for the details" >> $MIGLOG
}

#Run the DB2 database restore.
function RestoreDatabase
{
	echo "Database backup directory: $BACKUP_DIR" >> $MIGLOG
	echo "Restoring database $DB_NAME ..." >> $MIGLOG
#       Get latest file in backup directory if it is not the latest

        if [ -z "$BACKUPFILE" ] ; then
           BACKUPFILE=$(ls -t $BACKUP_DIR | head -1)
        fi

        chmod 644 $BACKUP_DIR/$BACKUPFILE

        if [ $TIMESTAMP ] ; then
           TAKEN_AT="TAKEN AT $TIMESTAMP"
        else
           TAKEN_AT=""
        fi
        su - $DB_USER -c "$DIRHOME/StorageControl/bin/SCMigRestoreSupport.sh $MIGRATION_PARAM_FILE $DIRHOME" >> $MIGLOG 2>&1
	migrationReturnCode=$?
}

#Connect to the database and unlock it to give control back to the system.
function CompleteMigration 
{

	# connect to db2, unquiesce database
	echo "$DB2_CMD connect TO $DB_NAME" >> $MIGLOG
	$DB2_CMD CONNECT TO $DB_NAME USER $DB_USER USING $DB_PASSWORD
	if [ "$?" -ne "0" ] ; then
	    exit 1 
	fi
	echo "Unquiesce database ..." >> $MIGLOG
	$DB2_CMD UNQUIESCE DATABASE
	$DB2_CMD CONNECT RESET
}

#Do these tasks for Backup operations.
echo "Migration operation is $MIGRATION_OPERATION" >> $MIGLOG
if [ "$MIGRATION_OPERATION" == "backup" ] ; then
   PrepareForMigration
   BackupDatabase
   CompleteMigration
fi

#Do these tasks for Restore operations.
if [ "$MIGRATION_OPERATION" == "restore" ] ; then
   RestoreDatabase  
   echo "Restore Completed - migrationCode - $migrationReturnCode" >> $MIGLOG
fi


#  We ignore a return of 2 for warnings from the restore operation
if [ $migrationReturnCode = "2" ] ; then
   if [ "$MIGRATION_OPERATION" == "restore" ] ; then
      exit 0  
   fi
fi

if [ $migrationReturnCode -ne "0" ] ; then
    exit 1
fi

echo "Returning Successfully" >> $MIGLOG
exit 0
